/******************************************************************************
* choria
* Copyright (c) 2023 Alan Witkowski
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/
#pragma once

// Libraries
#include <objects/statchange.h>
#include <constants.h>
#include <glm/vec2.hpp>
#include <vector>
#include <cstdint>

// Forward Declarations
class _Stats;
class _Object;
class _Item;
class _Buff;
class _StatusEffect;
struct _ActionResult;

namespace ae {
	class _Texture;
	class _Buffer;
}

// Types of targets
enum class TargetType : uint32_t {
	NONE,
	SELF,
	ENEMY,
	ALLY,
	ANY,
	ENEMY_ALL,
	ALLY_ALL,
	ENEMY_CORPSE_AOE,
};

// Scope of action
enum class ScopeType : uint8_t {
	NONE,
	WORLD,
	BATTLE,
	ALL
};

struct _Summon {
	const _Buff *SummonBuff{nullptr};
	double Duration{0.0};
	int64_t Health{0};
	int64_t Mana{0};
	uint32_t ID{0};
	uint32_t SpellID{0};
	int Armor{0};
	int Limit{0};
	int BattleSpeed{0};
	int SkillLevel{0};
	int MinDamage{0};
	int MaxDamage{0};
	int ResistAll{0};
};

struct _SummonCaptain {
	_Object *Owner{nullptr};
	std::vector<std::pair<_Summon, _StatusEffect *>> Summons;
};

class _Action {

	public:

		_Action() { }
		_Action(const _Item *Item) : Item(Item) { }

		bool operator==(const _Action &Action) const { return Action.Item == Item; }
		bool operator!=(const _Action &Action) const { return !(Action.Item == Item); }

		void Serialize(ae::_Buffer &Data);
		void Unserialize(ae::_Buffer &Data, const _Stats *Stats);

		bool Resolve(ae::_Buffer &Data, _Object *Source, ScopeType Scope);
		void HandleSummons(_ActionResult &ActionResult);

		bool IsSet() const { return !(Item == nullptr); }
		void Unset() { Item = nullptr; Count = 0; Duration = 0.0; Level = 0; InventorySlot = -1; ActionBarSlot = -1; }

		TargetType GetTargetType();

		const _Item *Item{nullptr};
		double Duration{0.0f};
		int Level{0};
		int Count{0};
		int InventorySlot{-1};
		int ActionBarSlot{-1};
};

struct _ActionResult {
	_StatChange Source;
	_StatChange Target;
	const _Buff *SummonBuff{nullptr};
	std::vector<_Summon> Summons;
	glm::vec2 LastPosition{0.0f};
	glm::vec2 Position{0.0f};
	_Action ActionUsed;
	const ae::_Texture *Texture{nullptr};
	double Time{0.0};
	double Timeout{HUD_ACTIONRESULT_TIMEOUT};
	double Speed{HUD_ACTIONRESULT_SPEED};
	bool Miss{false};
	ScopeType Scope{ScopeType::ALL};
};
