/******************************************************************************
* choria
* Copyright (c) 2023 Alan Witkowski
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/
#pragma once

// Libraries
#include <constants.h>
#include <glm/vec2.hpp>
#include <glm/vec4.hpp>
#include <unordered_map>
#include <string>
#include <cmath>

// Forward Declarations
class _Object;

namespace ae {
	template<class T> class _Manager;
	class _Font;
	class _Buffer;
}

enum class StatValueType : int {
	BOOLEAN,
	INTEGER,
	INTEGER64,
	FLOAT,
	PERCENT,
	TIME,
	POINTER,
};

enum class StatUpdateType : int {
	NONE,
	ADD,
	SET,
	MULTIPLICATIVE,
};

// Holds a value union
struct _Value {

	float Mult() const { return Int * 0.01f; }
	float BonusMult() const { return (100 + Int) * 0.01f; }
	int Multiplicative(int Current) { return std::ceil(Int * (100 - Current) * 0.01f); }
	static const _Value &Print(const std::unordered_map<std::string, _Value> &Map, const char *Key);

	union {
		int Int;
		int64_t	Int64;
		float Float;
		double Double;
		void *Pointer;
	};
};

// Stat changes
class _StatChange {

	public:

		void Reset() { Object = nullptr; Values.clear(); }
		bool HasStat(const std::string &Name) const { return Values.find(Name) != Values.end();	}

		void Serialize(ae::_Buffer &Data);
		void Unserialize(ae::_Buffer &Data, ae::_Manager<_Object> *Manager);

		// Owner
		_Object *Object{nullptr};

		// Data
		std::unordered_map<std::string, _Value> Values;
};

// Graphical stat change
class _StatChangeUI {

	public:

		_StatChangeUI() { }

		void Render(double BlendFactor);
		void SetText(const glm::vec4 &NegativeColor, const glm::vec4 &PositiveColor, bool UseLocale=false);

		const ae::_Font *Font{nullptr};
		std::string Text;
		glm::vec4 Color{1.0f};
		glm::vec2 StartPosition{0.0f};
		glm::vec2 LastPosition{0.0f};
		glm::vec2 Position{0.0f};
		float Direction{-1.0f};
		double Time{0.0};
		double Timeout{HUD_STATCHANGE_TIMEOUT};
		float Change{0.0f};
		bool Battle{false};

};
